﻿function OnInit(initData)
{
	initData.name = "EditBOM";
	initData.version = "1.0";
	initData.copyright = "(c) 2018 Leo Davidson";
	initData.url = "https://resource.dopus.com/t/editbom-command-to-add-remove-or-toggle-utf-8-byte-order-mark/29786";
	initData.desc = "Add or remove the UTF-8 byte-order-mark from files.";
	initData.default_enable = true;
	initData.min_version = "12.9.2";

	var cmd = initData.AddCommand();
	cmd.name = "EditBOM";
	cmd.method = "OnEditBOM";
	cmd.desc = "Add or remove the UTF-8 byte-order-mark from files.";
	cmd.label = "EditBOM";
	cmd.template = "FILE,ADD/S,REMOVE/S,TOGGLE/S";
	cmd.hide = false;
	cmd.icon = "edit";
}

// Helper for nicer loops over Opus collections.
function forEach(col, func)
{
	for (var e = new Enumerator(col); !e.atEnd(); e.moveNext())
	{
		if (!func(e.item()))
		{
			break;
		}
	}
}

// Implement the EditBOM command
function OnEditBOM(scriptCmdData)
{
	var cmd = scriptCmdData.func.Command;
	cmd.deselect = false;

	var filePath = scriptCmdData.func.args.file;
	var isAborted = false;

	if (!scriptCmdData.func.args.add
	&&  !scriptCmdData.func.args.remove
	&&  !scriptCmdData.func.args.toggle)
	{
		func.dlg.Request("You must specify ADD, REMOVE or TOGGLE.", "OK", "EditBOM");
		isAborted = true;
	}
	else
	{
		if (filePath)
		{
			if (!ProcessFile(scriptCmdData.func, filePath, false))
			{
				isAborted = true;
			}
		}
		else
		{
			forEach(scriptCmdData.func.sourcetab.selected_files, function(file)
			{
				if (!ProcessFile(scriptCmdData.func, file.RealPath, true))
				{
					isAborted = true;
					return false; // Break forEach loop.
				}
				return true; // Continue forEach loop.
			});
		}
	}

	// Results of Command functions are reversed to normal, due to returning success as default.
	// True here means the command failed, and Opus should not run further commands.
	// False (or no return at all) means the command succeeded, and Opus should continue with any other commands.
	if (isAborted)
		return true;
	return false;
}

function ProcessFile(func, filePath, doDeselect)
{
	if (!DOpus.FSUtil.Exists(filePath))
	{
		return (func.dlg.Request("File does not exist:\n" + filePath, "&Skip|&Abort", "EditBOM") == 1);
	}

	var blobBOM = DOpus.Create.Blob(0xEF,0xBB,0xBF);
	var blobExisting = DOpus.Create.Blob();

	var fileRead = DOpus.FSUtil.OpenFile(filePath, "r", func.sourcetab);

	if (fileRead.error != 0
	||	(fileRead.size.Compare(0) != 0 && fileRead.Read(blobExisting) <= 0))
	{
		return (func.dlg.Request("Could not read file:\n" + filePath, "&Skip|&Abort", "EditBOM") == 1);
	}

	fileRead.Close();

	// ActiveScripting limits the normal integer size to 2147483647 (2GB).
	// The Opus FileSize objects can handle more, but it makes life difficult since we cannot
	// easily pass things to them to compare sizes. Since a 2GB text file is ridiculous, we
	// simply reject such things here.
	//
	//		var maxSize = 2147483647 - 3; // -3 because we need space to add the BOM.
	//
	// In fact, we'll set the maximum much smaller, as the script as-is does not show a progress dialog
	// etc. which you would really want for converting larger files. Adding a progress dialog is
	// totally possible, but then you'll probably also want the ability to skip/abort in the middle
	// of writing a large file, in which case it would make sense to use temp-files instead.
	// (The script as it is simply overwrites the old files, which should be fine for small ones
	// but is less sensible for large amounts of data.)

	var maxSize = 10485760; // 10MB max size, for now.

	if (blobExisting.size.Compare(maxSize) > 0)
	{
		return (func.dlg.Request("File is too large:\n" + filePath, "&Skip|&Abort", "EditBOM") == 1);
	}

	var hasBOM = false;

	if (blobExisting.size.Compare( blobBOM.size ) >= 0
	&&	blobExisting.Compare( blobBOM, 0, 0, blobBOM.size ) == 0)
	{
		hasBOM = true;
	}

	if (( hasBOM && func.args.add)
	||  (!hasBOM && func.args.remove))
	{
		// No-Op. Just de-select it.
		if (doDeselect)
		{
			DeselectFile(func, filePath);
		}
		return true;
	}

	var addMode = false;

	if (func.args.add
	||	(func.args.toggle && !hasBOM))
	{
		addMode = true;
	}

	var errMsg = null;
	var fileWrite = null;

	// Retry Loop.
	while (true)
	{
		if (fileWrite)
		{
			fileWrite.Close();
			fileWrite = null;
		}

		if (errMsg)
		{
			var dlgRes = func.dlg.Request(errMsg, "&Retry|&Skip|&Abort", "EditBOM");
			if (dlgRes == 0)
				return false; // Abort
			if (dlgRes == 2)
				return true; // Skip
			// Else, dlgRes == 1, retry.
		}

		fileWrite = DOpus.FSUtil.OpenFile(filePath, "wa", func.sourcetab);
		if (fileWrite.error != 0)
		{
			errMsg = "Error opening file for writing (" + fileWrite.error + "):\n" + filePath;
			continue;
		}

		var writtenSize;

		if (addMode)
		{
			writtenSize = fileWrite.Write(blobBOM);
			if (blobBOM.size.Compare( writtenSize ) != 0)
			{
				errMsg = "Error writing to file:\n" + filePath;
				continue;
			}
		}

		var offsetBytes = (addMode ? 0 : 3);

		// Skip the write entirely if there's nothing to write.
		// (Faster, but also avoids an issue in Opus 12.9.x.)
		if (blobExisting.size.Compare(offsetBytes) > 0)
		{
			writtenSize = fileWrite.Write(blobExisting, offsetBytes);

			if (blobExisting.size.Compare( writtenSize + offsetBytes ) != 0)
			{
				errMsg = "Error writing to file:\n" + filePath;
				continue;
			}
		}

		if (doDeselect)
		{
			DeselectFile(func, filePath);
		}
		return true; // Success for this file.
	}
}

function DeselectFile(func, filePath)
{
	var cmd = func.Command;
	cmd.ClearFiles(); // Just in case.
	cmd.AddFile(filePath);
	cmd.RunCommand("Select DESELECT FROMSCRIPT");
	cmd.ClearFiles();
}
